import typer

from flask_management.constants import PythonVersion
from flask_management.context import ProjectContext
from flask_management.generator import generate_project

app = typer.Typer(
    add_completion=False,
    help="Managing flask projects made easy!",
    name="Manage Flask",
)


@app.command(help="Creates a flask project.")
def startproject(
    name: str,
    python: PythonVersion = typer.Option(PythonVersion.THREE_DOT_EIGHT),
):
    context = ProjectContext(
        name=name,
        python=python
    )
    generate_project(context)



from blinker import Namespace

{{ cookiecutter.app_name }}_signals = Namespace()

booting = {{ cookiecutter.app_name }}_signals.signal("BOOTING")
after_boot = {{ cookiecutter.app_name }}_signals.signal("AFTER-BOOT")



from {{ cookiecutter.app_name }}.signals import after_boot, booting
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy

db = SQLAlchemy()


@booting.connect
def init(app):
    db.init_app(app)


@after_boot.connect
def init_cmd(app):
    Migrate(app, db)



import os
from typing import TypeVar

import typer
from cookiecutter.exceptions import OutputDirExistsException
from cookiecutter.main import cookiecutter
from pydantic.main import BaseModel

from flask_management.constants import TEMPLATES_DIR
from flask_management.context import ProjectContext

ContextType = TypeVar("ContextType", bound=BaseModel)


def fill_template(template_name: str, context: ContextType):
    try:
        cookiecutter(
            os.path.join(TEMPLATES_DIR, template_name),
            extra_context=context.dict(),
            no_input=True,
        )
    except OutputDirExistsException:
        typer.echo(f"Folder '{context.folder_name}' already exists. 😞")
    else:
        typer.echo(f"Flask {template_name} created successfully! 🎉")


def generate_project(context: ProjectContext):
    fill_template("project", context)



from flask import Flask

from {{ cookiecutter.app_name }}.app_settings import AppSettings
from {{ cookiecutter.app_name }}.signals import booting, after_boot

from {{ cookiecutter.app_name }} import ext
from {{ cookiecutter.app_name }} import models
from {{ cookiecutter.app_name }} import book


app = Flask(__name__)
settings = AppSettings()

app.config.from_object(settings)

booting.send(app)
after_boot.send(app)



import os
from enum import Enum, EnumMeta

TEMPLATES_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "templates")


class BaseMetadataEnum(EnumMeta):
    def __contains__(self, other):
        try:
            self(other)
        except ValueError:
            return False
        else:
            return True


class BaseEnum(str, Enum, metaclass=BaseMetadataEnum):
    """Base enum class."""


class PythonVersion(BaseEnum):
    THREE_DOT_SEVEN = "3.7"
    THREE_DOT_EIGHT = "3.8"
    THREE_DOT_NINE = "3.9"
    THREE_DOT_TEN = "3.10"
    THREE_DOT_ELEVEN = "3.11"


class Database(BaseEnum):
    POSTGRES = "Postgres"
    MYSQL = "MySQL"



from {{ cookiecutter.app_name }}.ext import db


class Book(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    created_at = db.Column(
        db.DateTime(True), nullable=False,
        server_default=db.text("CURRENT_TIMESTAMP")
    )
    updated_at = db.Column(
        db.DateTime(True), nullable=False,
        server_default=db.text("CURRENT_TIMESTAMP")
    )
    name = db.Column(db.String(64))



import subprocess
from datetime import datetime
from typing import Optional

from pydantic import BaseModel, EmailStr, root_validator

from flask_management.constants import PythonVersion


class ProjectContext(BaseModel):
    name: str
    folder_name: Optional[str]
    app_name: Optional[str]

    username: Optional[str] = None
    email: Optional[EmailStr] = None

    python: PythonVersion
    year: Optional[int]

    @root_validator(pre=True)
    def validate_project(cls, values: dict):
        try:
            values["username"] = subprocess.check_output(
                ["git", "config", "--get", "user.name"]
            )
            values["email"] = subprocess.check_output(
                ["git", "config", "--get", "user.email"]
            )
        except subprocess.CalledProcessError:
            ...
        values["folder_name"] = values["name"].lower().replace(" ", "-").strip()
        values["year"] = datetime.today().year
        values["app_name"] = values["folder_name"].replace("-", "_").strip()
        return values

    class Config:
        use_enum_values = True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flask_management',
 'flask_management.templates',
 'flask_management.templates.project',
 'flask_management.templates.project.{{ cookiecutter.folder_name }}.tests',
 'flask_management.templates.project.{{ cookiecutter.folder_name }}.{{ '
 'cookiecutter.app_name }}',
 'flask_management.templates.project.{{ cookiecutter.folder_name }}.{{ '
 'cookiecutter.app_name }}.book']

package_data = \
{'': ['*'],
 'flask_management.templates.project': ['{{ cookiecutter.folder_name }}/*']}

install_requires = \
['cookiecutter>=1.7.2', 'pydantic[email]>=1.7.2', 'typer']

entry_points = \
{'console_scripts': ['flaskapi = flask_management.main:app']}

setup_kwargs = {
    'name': 'flask-management',
    'version': '0.1.2',
    'description': 'create the flask project quickly.',
    'long_description': '# flask-management\n\n## How to use :\n\nflaskapi your-project-name\n',
    'author': 'huangsong',
    'author_email': 'huangsong@leyantech.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ponytailer/flask-management',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pydantic import BaseSettings


class AppSettings(BaseSettings):
    SECRET_KEY: str
    SQLALCHEMY_DATABASE_URI: str

    SQLALCHEMY_ECHO: bool = False
    SQLALCHEMY_TRACK_MODIFICATIONS: bool = False
